package ru.dbotthepony.kstarbound.defs.actor

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableSet
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.defs.AssetPath
import ru.dbotthepony.kstarbound.defs.ColorReplacements
import ru.dbotthepony.kstarbound.defs.StatusEffectDefinition
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.defs.actor.player.BlueprintLearnList
import ru.dbotthepony.kstarbound.defs.item.ItemDescriptor
import ru.dbotthepony.kstarbound.json.builder.JsonFactory

@JsonFactory
data class Species(
	val kind: String,
	val charCreationTooltip: Tooltip = Tooltip(),
	val nameGen: ImmutableList<String>,
	val ouchNoises: ImmutableList<AssetPath>,
	val charGenTextLabels: ImmutableList<String> = ImmutableList.of(),
	val skull: AssetPath = AssetPath("/humanoid/any/dead.png"), // ваш свет угасает
	val defaultBlueprints: BlueprintLearnList,

	val headOptionAsHairColor: Boolean = false,
	val headOptionAsFacialhair: Boolean = false,
	val altOptionAsUndyColor: Boolean = false,
	val altOptionAsHairColor: Boolean = false,
	val altOptionAsFacialMask: Boolean = false,
	val hairColorAsBodySubColor: Boolean = false,
	val bodyColorAsFacialMaskSubColor: Boolean = false,
	val altColorAsFacialMaskSubColor: Boolean = false,

	val bodyColor: ImmutableList<ColorReplacements>,
	val undyColor: ImmutableList<ColorReplacements>,
	val hairColor: ImmutableList<ColorReplacements>,
	val genders: ImmutableList<GenderSettings>,
	val statusEffects: ImmutableSet<Registry.Ref<StatusEffectDefinition>> = ImmutableSet.of(),
) {
	@JsonFactory
	data class Tooltip(val title: String = "", val subTitle: String = "", val description: String = "")

	@JsonFactory
	data class GenderSettings(
		val name: String = "",
		val image: SpriteReference = SpriteReference.NEVER,
		val characterImage: SpriteReference = SpriteReference.NEVER,
		val hairGroup: String? = null,
		val hair: ImmutableSet<String>,
		val shirt: ImmutableSet<ItemDescriptor>,
		val pants: ImmutableSet<ItemDescriptor>,
		val facialHairGroup: String? = null,
		val facialHair: ImmutableSet<String> = ImmutableSet.of(),
		val facialMaskGroup: String? = null,
		val facialMask: ImmutableList<String> = ImmutableList.of(),
	)
}
