package ru.dbotthepony.kstarbound.defs.world

import com.google.common.collect.ImmutableSet
import com.google.gson.JsonObject
import com.google.gson.JsonPrimitive
import ru.dbotthepony.kommons.gson.contains
import ru.dbotthepony.kstarbound.Globals
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.fromJsonFast
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import java.util.random.RandomGenerator

@JsonFactory
data class SpawnProfile(val spawnTypes: ImmutableSet<Registry.Ref<SpawnType>> = ImmutableSet.of(), val monsterParameters: JsonObject = JsonObject()) {
	companion object {
		fun create(json: JsonObject, random: RandomGenerator): SpawnProfile {
			val spawnTypes = ArrayList<Registry.Ref<SpawnType>>()

			if ("groups" in json) {
				for (group in json["groups"].asJsonArray) {
					val pool = group.asJsonObject["pool"]
					val select = group.asJsonObject["select"].asJsonPrimitive.asInt

					val typePool = if (pool is JsonPrimitive) {
						Globals.spawner.spawnGroups[pool.asString] ?: throw NoSuchElementException("No such common spawn group with name $pool")
					} else {
						Starbound.gson.fromJsonFast(pool)
					}

					spawnTypes.addAll(typePool.sample(select, random))
				}
			}

			return SpawnProfile(ImmutableSet.copyOf(spawnTypes), json["monsterParameters"] as? JsonObject ?: JsonObject())
		}
	}
}

