package ru.dbotthepony.kstarbound.world.entities.behavior

import com.google.common.collect.ImmutableList
import com.google.gson.JsonPrimitive
import org.classdump.luna.runtime.ExecutionContext
import ru.dbotthepony.kstarbound.defs.actor.behavior.NodeParameter
import ru.dbotthepony.kstarbound.lua.userdata.BehaviorState

class ParallelNode(parameters: Map<String, NodeParameter>, val children: ImmutableList<AbstractBehaviorNode>) : AbstractBehaviorNode() {
	val successLimit: Int
	val failLimit: Int

	init {
		val value = (parameters["success"]?.value?.value as? JsonPrimitive)?.asInt ?: -1

		if (value == -1) {
			successLimit = children.size
		} else {
			successLimit = value
		}
	}

	init {
		val value = (parameters["fail"]?.value?.value as? JsonPrimitive)?.asInt ?: -1

		if (value == -1) {
			failLimit = children.size
		} else {
			failLimit = value
		}
	}

	private var lastFailed = -1
	private var lastSucceeded = -1

	override fun run(delta: Double, state: BehaviorState): Status {
		var failed = 0
		var succeeded = 0

		for (node in children) {
			val status = node.runAndReset(delta, state)

			if (status == Status.SUCCESS)
				succeeded++
			else if (status == Status.FAILURE)
				failed++

			if (succeeded >= successLimit || failed >= failLimit) {
				lastFailed = failed
				lastSucceeded = succeeded
				return if (succeeded >= successLimit) Status.SUCCESS else Status.FAILURE
			}
		}

		lastFailed = failed
		lastSucceeded = succeeded
		return Status.RUNNING
	}

	override fun toString(): String {
		return "ParallelNode[children=${children.size}, successLimit=$successLimit, failLimit=$failLimit, lastFailed=$lastFailed, lastSucceeded=$lastSucceeded]"
	}

	override fun reset() {
		children.forEach { it.reset() }
		lastSucceeded = -1
		lastFailed = -1
	}
}
